@@ -23,7 +23,7 @@ from portage.util._dyn_libs.soname_deps_qa import (
 )
 from portage.package.ebuild.prepare_build_dirs import (_prepare_workdir,
 		_prepare_fake_distdir, _prepare_fake_filesdir)
-from portage.util.futures.compat_coroutine import coroutine, coroutine_return
+from portage.util.futures.compat_coroutine import coroutine
 from portage.util import writemsg
 from portage.util._async.AsyncTaskFuture import AsyncTaskFuture
 from portage.util._async.BuildLogger import BuildLogger
@@ -71,7 +71,8 @@ class EbuildPhase(CompositeTask):
 	_locked_phases = ("setup", "preinst", "postinst", "prerm", "postrm")
 
 	def _start(self):
-		self.scheduler.run_until_complete(self._async_start())
+		future = asyncio.ensure_future(self._async_start(), loop=self.scheduler)
+		self._start_task(AsyncTaskFuture(future=future), self._async_start_exit)
 
 	@coroutine
 	def _async_start(self):
@@ -140,21 +141,30 @@ class EbuildPhase(CompositeTask):
 					os.path.join(self.settings['PKGDIR'],
 					self.settings['CATEGORY'], self.settings['PF']) + '.tbz2'
 
+	def _async_start_exit(self, task):
+		task.future.cancelled() or task.future.result()
+		if self._default_exit(task) != os.EX_OK:
+			self.wait()
+			return
+
 		if self.phase in ("pretend", "prerm"):
 			env_extractor = BinpkgEnvExtractor(background=self.background,
 				scheduler=self.scheduler, settings=self.settings)
 			if env_extractor.saved_env_exists():
-				self._current_task = env_extractor
-				yield env_extractor.async_start()
-				yield env_extractor.async_wait()
-				if self._default_exit(env_extractor) != os.EX_OK:
-					self._async_wait()
-					coroutine_return()
+				self._start_task(env_extractor, self._env_extractor_exit)
+				return
 			# If the environment.bz2 doesn't exist, then ebuild.sh will
 			# source the ebuild as a fallback.
 
 		self._start_lock()
 
+	def _env_extractor_exit(self, env_extractor):
+		if self._default_exit(env_extractor) != os.EX_OK:
+			self.wait()
+			return
+
+		self._start_lock()
+
 	def _start_lock(self):
 		if (self.phase in self._locked_phases and
 			"ebuild-locks" in self.settings.features):
@@ -422,7 +432,7 @@ class EbuildPhase(CompositeTask):
 						log_path=log_path,
 						log_filter_file=self.settings.get('PORTAGE_LOG_FILTER_FILE_CMD'),
 						scheduler=self.scheduler)
-					yield build_logger.async_start()
+					build_logger.start()
 					log_file = build_logger.stdin
 
 				yield self.scheduler.async_output(msg, log_file=log_file,
