@@ -45,10 +45,7 @@ class SpawnProcess(SubProcess):
 	_CGROUP_CLEANUP_RETRY_MAX = 8
 
 	def _start(self):
-		self.scheduler.run_until_complete(self._async_start())
 
-	@coroutine
-	def _async_start(self):
 		if self.fd_pipes is None:
 			self.fd_pipes = {}
 		else:
@@ -147,31 +144,21 @@ class SpawnProcess(SubProcess):
 			log_path=log_file_path,
 			log_filter_file=self.log_filter_file,
 			scheduler=self.scheduler)
+		build_logger.start()
+
+		pipe_logger = PipeLogger(background=self.background,
+			scheduler=self.scheduler, input_fd=master_fd,
+			log_file_path=build_logger.stdin,
+			stdout_fd=stdout_fd)
+
+		pipe_logger.start()
 
 		self._registered = True
-		pipe_logger = None
-		try:
-			yield build_logger.async_start()
-
-			pipe_logger = PipeLogger(background=self.background,
-				scheduler=self.scheduler, input_fd=master_fd,
-				log_file_path=build_logger.stdin,
-				stdout_fd=stdout_fd)
-
-			yield pipe_logger.async_start()
-		except asyncio.CancelledError:
-			if pipe_logger is not None and pipe_logger.poll() is None:
-				pipe_logger.cancel()
-			if build_logger.poll() is None:
-				build_logger.cancel()
-			raise
-
-		self._main_task = asyncio.ensure_future(
-			self._main(pipe_logger, build_logger), loop=self.scheduler)
+		self._main_task = asyncio.ensure_future(self._main(build_logger, pipe_logger), loop=self.scheduler)
 		self._main_task.add_done_callback(self._main_exit)
 
 	@coroutine
-	def _main(self, pipe_logger, build_logger):
+	def _main(self, build_logger, pipe_logger):
 		try:
 			if pipe_logger.poll() is None:
 				yield pipe_logger.async_wait()
@@ -222,9 +209,9 @@ class SpawnProcess(SubProcess):
 			self._main_task.done() or self._main_task.cancel()
 
 	def _cancel(self):
-		SubProcess._cancel(self)
 		if self._main_task is not None:
 			self._main_task.done() or self._main_task.cancel()
+		SubProcess._cancel(self)
 		self._cgroup_cleanup()
 
 	def _cgroup_cleanup(self):
