@@ -87,8 +87,9 @@ class PipeLogger(AbstractPollTask):
 				try:
 					yield future
 				finally:
-					self.scheduler.remove_reader(fd)
-					future.done() or future.cancel()
+					if not self.scheduler.is_closed():
+						self.scheduler.remove_reader(fd)
+						future.done() or future.cancel()
 				continue
 
 			if not buf:
@@ -130,13 +131,19 @@ class PipeLogger(AbstractPollTask):
 								fcntl.F_GETFL) ^ os.O_NONBLOCK)
 
 				if log_file is not None:
+					if isinstance(log_file, gzip.GzipFile):
+						# Use log_file.write since data written directly
+						# to the file descriptor bypasses compression.
+						log_file.write(buf)
+						log_file.flush()
+						continue
+
 					write_buf = buf
-					while True:
+					while write_buf:
 						try:
-							if write_buf is not None:
-								log_file.write(write_buf)
-								write_buf = None
-							log_file.flush()
+							# Use os.write, since the log_file.write method
+							# looses data when an EAGAIN occurs.
+							write_buf = write_buf[os.write(log_file.fileno(), write_buf):]
 						except EnvironmentError as e:
 							if e.errno != errno.EAGAIN:
 								raise
@@ -145,10 +152,9 @@ class PipeLogger(AbstractPollTask):
 							try:
 								yield future
 							finally:
-								self.scheduler.remove_writer(self._log_file.fileno())
-								future.done() or future.cancel()
-						else:
-							break
+								if not self.scheduler.is_closed():
+									self.scheduler.remove_writer(self._log_file.fileno())
+									future.done() or future.cancel()
 
 	def _io_loop_done(self, future):
 		try:
